oneStep :: Fields -> GUI   ()
oneStep(fields) = do
   liftIO   (fire1 gameOfLife)
   seqs   [ cset f   (background (color c)) | (f,c) <- fields ]

type Field    = (Button,Cell)
type Fields   = [Field] 

color :: Cell -> String
color cell =  
  if status cell == Dead then "lightgrey" else "red"

toggle :: Field -> GUI   ()
toggle(b,cell) = do
   liftIO   (fire1 (if status cell == Dead then
                      status cell := Alive
                    else
                      status cell := Dead)) 
   cset b   (background (color cell))

main :: IO   ()
main = start   $   do
   w   <- window   [title "Game of Life"]
   q   <- button   [text "quit", command quit]   w
   bfire   <- button   [text "fire"]   w
   fields   <- binds   [ button [background (color c),  
                              font "courier"]   w  
                       | c <- cells ] 
   let fields' = zip fields cells
   seqs   [cset f (command (toggle(f,c)))  
	  | (f,c) <- fields']
   cset bfire   (command (oneStep fields'))
   pack   (matrix cN fields   ^^   fillX bfire   ^^   fillX q)
   controlWindow(fields')

controlWindow :: Fields -> GUI   ()
controlWindow(fields) = do
   w    <- window   [title "speed control"]
   t    <- timer   [initValue   1000, active False,  
                    command (oneStep(fields)) ]
   bc   <- button   [text "cancel", active False]   w
   br   <- button   [text "run"]   w
   s    <- hscale   [ scaleRange   (0,3000),  
                      tickInterval   1000,
                      text "delay in ms",  
                      height   200,  
                      initValue   1000 ]   w 
   pack   (flexible (flexible s ^^ bc << br))
   cset s    (command (do v <- getValue s; setValue t v))
   cset br   (command (animation(t,s,bc,br,fields)))
   cset bc   (command (do cset t   (active False)
                          cset br   (active True)
                          cset bc   (active False))) 

animation :: (Timer,Scale, Button, Button, Fields) -> GUI   ()
animation(t,s,bc,br,fields) = do
  cset br   (active False)
  cset bc   (active True)
  cset t    (active True) 
